<?php
/**
 * @package Nicepage Website Builder
 * @author Nicepage https://www.nicepage.com
 * @copyright Copyright (c) 2016 - 2019 Nicepage
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */
defined('_JEXEC') or die;

/**
 * Class NicepageHelpersNicepage
 */
class NicepageHelpersNicepage
{
    /**
     * Extension type name
     *
     * @var string
     */
    public static $extension = 'com_nicepage';

    /**
     * Add submenu on page
     *
     * @param string $vName Page name
     */
    public static function addSubmenu($vName)
    {
        JHtmlSidebar::addEntry(
            JText::_('COM_NICEPAGE_SUBMENU_EDITOR'),
            'index.php?option=com_nicepage&task=nicepage.start',
            false
        );

        JHtmlSidebar::addEntry(
            JText::_('COM_NICEPAGE_SUBMENU_IMPORT'),
            'index.php?option=com_nicepage&view=import',
            $vName == 'import'
        );

        JHtmlSidebar::addEntry(
            JText::_('COM_NICEPAGE_SUBMENU_CONFIGURATION'),
            'index.php?option=com_nicepage&view=config',
            $vName == 'config'
        );

        JHtmlSidebar::addEntry(
            JText::_('COM_NICEPAGE_SUBMENU_THEME'),
            'index.php?option=com_nicepage&view=theme',
            false
        );
    }

    /**
     * Get actions for nicepage component
     *
     * @return JObject
     */
    public static function getActions()
    {
        $user = JFactory::getUser();
        $result = new JObject;

        $assetName = 'com_nicepage';
        $level = 'component';

        $actions = JAccess::getActions('com_nicepage', $level);

        foreach ($actions as $action) {
            $result->set($action->name, $user->authorise($action->name, $assetName));
        }

        return $result;
    }

    /**
     * Get domain get parameter
     *
     * @return mixed|string
     */
    public static function getDomain()
    {
        $default = defined('NICEPAGE_DOMAIN') ? NICEPAGE_DOMAIN : '';
        $app = JFactory::getApplication();
        $domain = urldecode($app->input->getVar('domain', $default));
        if ($domain) {
            $domain = preg_replace('#^https?:#', '', $domain); // remove protocol
            $domain = preg_replace('#\/$#', '', $domain); // remove last slash
        }
        return $domain;
    }

    /**
     * Get site url
     *
     * @return string
     */
    public static function getSiteUrl()
    {
        return dirname(dirname((JURI::current())));
    }

    /**
     * Get files for nicepage starting
     *
     * @return array
     */
    public static function getStartFiles()
    {
        $domain = self::getDomain();

        $extension = JTable::getInstance('extension');
        $id = $extension->find(array('element' => 'com_nicepage'));
        $extension->load($id);
        $componentInfo = json_decode($extension->manifest_cache, true);
        $hash = $componentInfo['version'];

        return array(
            'sw' => self::getSiteUrl() . '/administrator/index.php?option=com_nicepage&task=actions.getSw',
            'editor' => self::getSiteUrl() . '/administrator/components/com_nicepage/assets/app/editor.js?ver=' . $hash,
            'loader' => $domain ? $domain . '/Editor/loader.js' : self::getLoader($hash),
            'auth' => self::getSiteUrl() . '/administrator/components/com_nicepage/helpers/auth.php?uid=' . JFactory::getUser()->id . '&ver=' . $hash
        );
    }

    /**
     * Get custom loader file
     *
     * @param string $hash Hash for get parameter
     *
     * @return string
     */
    public static function getLoader($hash) {
        return self::getSiteUrl() . '/administrator/components/com_nicepage/assets/app/loader.js?ver=' . $hash;
    }

    /**
     * Get actions list for nicepage app
     *
     * @return array
     */
    public static function getEditorSettings()
    {
        $current = dirname(dirname((JURI::current())));
        $index = $current . '/administrator/index.php?option=com_nicepage&task=actions.';
        return array(
            'actions' => array(
                'uploadFile' => $index . 'uploadFile',
                'uploadImage' => $index . 'uploadImage',
                'savePage' => $index . 'savePage',
                'clearChunks' => $index . 'clearChunks',
                'getSite' => $index . 'getSite',
                'getSitePosts' => $index . 'getSitePosts',
                'getPage' => $index . 'getPage',
                'saveSiteSettings' => $index . 'saveSiteSettings',
                'savePreferences' => $index . 'savePreferences',
                'saveMenuItems' => $index . 'saveMenuItems',
                'getPosts' => $current . '/index.php?option=com_nicepage&task=posts',
            ),
            'uploadFileOptions' => array(
                'formFileName' => 'async-upload'
            ),
            'dashboardUrl' => $current . '/administrator/',
            'editPostUrl' => $current . '/administrator/index.php?option=com_content&view=article&layout=edit&id={id}'
        );
    }

    /**
     * Get max request size
     *
     * @return mixed
     */
    public static function getMaxRequestSize()
    {
        $postSize = self::toBytes(ini_get('post_max_size'));
        $uploadSize = self::toBytes(ini_get('upload_max_filesize'));
        $memorySize = self::toBytes(ini_get('memory_limit'));

        return min($postSize, $uploadSize, $memorySize);
    }

    /**
     * Option value to bytes value
     *
     * @param string $str Option value
     *
     * @return int
     */
    public static function toBytes($str)
    {
        $str = strtolower(trim($str));
        $size = intval($str);
        if ($str && strlen($size) !== strlen($str)) {
            $unit = $str[strlen($str) - 1];
            $size = substr($str, 0, -1);
            switch ($unit) {
            case 'g':
                $size *= 1024;
            case 'm':
                $size *= 1024;
            case 'k':
                $size *= 1024;
            }
        }
        return $size;
    }

    /**
     * Get cms custom settings
     *
     * @return array
     */
    public static function getCmsSettings()
    {
        return array(
            'defaultImageUrl' => dirname(dirname((JURI::current()))) . '/components/com_nicepage/assets/images/nicepage-images/default-image.jpg',
            'defaultLogoUrl' => dirname(dirname((JURI::current()))) . '/components/com_nicepage/assets/images/nicepage-images/default-logo.png',
            'isFirstStart' => false,
            'maxRequestSize' => self::getMaxRequestSize(),
            'isWhiteLabelPlugin' => pathinfo(dirname(dirname(__FILE__)), PATHINFO_BASENAME) != ('com_' . 'n' . 'i' . 'c' . 'e' . 'p' . 'a' . 'g' . 'e')
        );
    }

    /**
     * Build site style css
     *
     * @param array  $params         plugin config parameters
     * @param string $pageCssUsedIds used css ids
     * @param string $html           public html of page
     * @param string $pageId         id of page
     *
     * @return string
     */
    public static function buildSiteStyleCss($params, $pageCssUsedIds, $html, $pageId)
    {
        $result = isset($params['siteStyleCss']) ? $params['siteStyleCss'] : '';
        $partsJson = isset($params['siteStyleCssParts']) ? $params['siteStyleCssParts'] : '';
        if ($partsJson) {
            $cssParts = json_decode($partsJson, true);
            if (!$pageCssUsedIds) {
                $usedIds = self::processUsedColors($cssParts, $html);
                self::updateUsedColor($usedIds, $pageId);
            } else {
                $usedIds = json_decode($pageCssUsedIds, true);
            }
            $headerFooterCssUsedIds = isset($params['headerFooterCssUsedIds']) ? json_decode($params['headerFooterCssUsedIds'], true) : [];
            $cookiesCssUsedIds = isset($params['cookiesCssUsedIds']) ? json_decode($params['cookiesCssUsedIds'], true) : [];
            $result   = '';
            foreach ($cssParts as $part) {
                if ($part['type'] !== 'color' || !empty($usedIds[$part['id']]) || !empty($headerFooterCssUsedIds[$part['id']]) || !empty($cookiesCssUsedIds[$part['id']])) {
                    $result .= $part['css'];
                }
            }
        }
        return $result;
    }

    /**
     * @param string $styles                common styles
     * @param string $publishHtml           publish html of page
     * @param string $publishHeaderFooter   publish html of page
     * @param string $publishCookiesSection publish cookies section
     *
     * @return array
     */
    public static function processAllColors($styles, $publishHtml, $publishHeaderFooter = '', $publishCookiesSection = '') {
        $split = preg_split('#(\/\*begin-color [^*]+\*\/[\s\S]*?\/\*end-color [^*]+\*\/)#', $styles, -1, PREG_SPLIT_DELIM_CAPTURE);
        $parts = array();
        foreach ($split as $part) {
            $part = trim($part);
            if (!$part) {
                continue;
            }

            if (preg_match('#\/\*begin-color ([^*]+)\*\/#', $part, $m)) {
                $id = 'color_' . $m[1];
                $parts[] = array(
                    'type' => 'color',
                    'id' => $id,
                    'css' => $part,
                );
                if (strpos($publishHtml, $m[1]) !== false) {
                    $usedIds[$id] = true;
                }
            } else {
                $parts[] = array(
                    'type' => '',
                    'css' => $part,
                );
            }
        }
        $headerFooterUsedIds = $publishHeaderFooter ? self::processUsedColors($parts, $publishHeaderFooter) : [];
        $cookiesCssUsedIds = $publishCookiesSection ? self::processUsedColors($parts, $publishCookiesSection) : [];
        $usedIds = self::processUsedColors($parts, $publishHtml);
        return array(json_encode($parts), json_encode($usedIds), json_encode($headerFooterUsedIds), json_encode($cookiesCssUsedIds));
    }

    /**
     * @param array  $parts       All styles array
     * @param string $publishHtml Html of page
     *
     * @return array
     */
    public static function processUsedColors($parts, $publishHtml)
    {
        $usedIds = array();
        foreach ($parts as &$part) {
            if (isset($part['id']) && strpos($publishHtml, preg_replace('#^color_#', '', $part['id'])) !== false) {
                $usedIds[$part['id']] = true;
            }
        }
        return $usedIds;
    }

    /**
     * Get nicepage properties
     *
     * @param boolean $isPreview Preview flag
     *
     * @return mixed
     */
    public static function getConfig($isPreview = false)
    {
        $ret = NicepageHelpersNicepage::getParamsTable()->getParameters();
        if (isset($ret['siteStyleCss']) && substr($ret['siteStyleCss'], 0, 6) === '<style') {
            // backward compatibility
            $css = preg_replace('#</?style[^>]*>#', '', $ret['siteStyleCss']);
            ob_start();
            include_once dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/assets/css/nicepage-dynamic.css';
            $dynamicPart = ob_get_clean();
            $ret['siteStyleCss'] = $css . $dynamicPart;
        }
        $ret['header'] = isset($ret['header']) ? $ret['header'] : '';
        $ret['footer'] = isset($ret['footer']) ? $ret['footer'] : '';
        if ($isPreview) {
            $ret['header'] = isset($ret['header:preview']) && $ret['header:preview'] ? $ret['header:preview'] : $ret['header'];
            $ret['footer'] = isset($ret['footer:preview']) && $ret['footer:preview'] ? $ret['footer:preview'] : $ret['footer'] ;
        }
        return $ret;
    }

    /**
     * Save nicepage settings
     *
     * @param array $data Data parameters
     *
     * @return mixed|string
     */
    public static function saveConfig($data)
    {
        $paramsTable = NicepageHelpersNicepage::getParamsTable();

        $params = $paramsTable->getParameters();
        $excludeParameters = array('option', 'action', 'controller', 'task', 'view');
        foreach ($data as $key => $value) {
            if (in_array($key, $excludeParameters)) {
                continue;
            }
            $params[$key] = $value;
        }
        $paramsTable->saveParameters($params);
    }

    /**
     * Get new sections object
     *
     * @return mixed
     */
    public static function getSectionsTable()
    {
        JLoader::register('PagesTableSections', dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/tables/sections.php');
        return JTable::getInstance('Sections', 'PagesTable');
    }

    /**
     * Get new params object
     *
     * @return mixed
     */
    public static function getParamsTable()
    {
        JLoader::register('PagesTableParams', dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/tables/params.php');
        return JTable::getInstance('Params', 'PagesTable');
    }

    /**
     * Clear preview page
     *
     * @param object $page sections page object
     *
     * @return null
     */
    public static function clearPreview($page) {
        $page->save(array('preview_props' => ''));
    }

    /**
     * @param array $parts  Used color list
     * @param int   $pageId Id of page
     */
    public static function updateUsedColor($parts, $pageId) {
        $page = NicepageHelpersNicepage::getSectionsTable();
        if ($page->load(array('page_id' => $pageId))) {
            $props = $page->props;
            $props['pageCssUsedIds'] = json_encode($parts);
            $page->save(array('props' => $props));
        }
    }

    /**
     * @param object $sectionsObj sections page object
     *
     * @return mixed|string
     */
    public static function getSectionsScreenshots($sectionsObj)
    {
        if (!$sectionsObj) {
            return '';
        }

        $props = $sectionsObj->props;
        $publishHtml = isset($props['publishHtml']) ? $props['publishHtml'] : '';
        $bodyClass = isset($props['bodyClass']) ? $props['bodyClass'] : '';
        $bodyStyle = isset($props['bodyStyle']) ? $props['bodyStyle'] : '';
        $head = isset($props['head']) ? $props['head'] : '';
        $fonts = isset($props['fonts']) ? $props['fonts'] : '';
        $pageCssUsedIds = isset($props['pageCssUsedIds']) ? $props['pageCssUsedIds'] : '';
        $publishHtml = self::processSectionsHtml($publishHtml, false);
        preg_match_all('/<section[\s\S]+?<\/section>/', $publishHtml, $matches, PREG_SET_ORDER);
        $count = count($matches);
        if ($count > 4) {
            for ($i = 4; $i < $count; $i++) {
                $publishHtml = str_replace($matches[$i], '', $publishHtml);
            }
        }

        $nicepageCss = JURI::root(true) . '/components/com_nicepage/assets/css/nicepage.css';

        include_once  dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/helpers/nicepage.php';
        $params = NicepageHelpersNicepage::getConfig();
        $siteStyleCss = NicepageHelpersNicepage::buildSiteStyleCss($params, $pageCssUsedIds, $publishHtml, $sectionsObj->page_id);

        $ret = <<<EOF
<!DOCTYPE html>
<html>        
    <head>
    <style>
        body {
            cursor: pointer;
        }
    </style>
    <link rel="stylesheet" href="$nicepageCss">
    $fonts
    <style>
    	$siteStyleCss
        $head
    </style>
    </head>
    <body class="$bodyClass" style="$bodyStyle">
        $publishHtml
    </body>
</html>
EOF;
        return $ret;
    }

    /**
     * @param string $content  Page content
     * @param bool   $isPublic Flag for public content
     * @param string $pageId   Type Id
     *
     * @return mixed
     */
    public static function processSectionsHtml($content, $isPublic = true, $pageId = '') {
        $content = self::processDefaultImage($content);
        if ($isPublic) {
            $content = self::processForm($content, $pageId);
            $content = self::processCustomPhp($content);
            $content = self::processControls($content);
            $content = self::processBlog($content);
        }
        $content = self::processPositions($content);
        return $content;
    }

    public static $posts = array();
    public static $post = array();

    /**
     * Process blog
     *
     * @param string $content Content
     *
     * @return string|string[]|null
     */
    public static function processBlog($content) {
        JLoader::register('Nicepage_Data_Mappers', JPATH_ADMINISTRATOR . '/components/com_nicepage/tables/mappers.php');
        JLoader::register('ContentModelCustomArticles', dirname(JPATH_PLUGINS) . '/components/com_nicepage/models/articles.php');
        $content = preg_replace_callback(
            '/<\!--blog-->([\s\S]+?)<\!--\/blog-->/',
            function ($blogMatch) {
                $blogHtml = $blogMatch[1];
                $blogOptions = [];
                if (preg_match('/<\!--blog_options_json--><\!--([\s\S]+?)--><\!--\/blog_options_json-->/', $blogHtml, $matches)) {
                    $blogOptions = json_decode($matches[1], true);
                    $blogHtml = str_replace($matches[0], '', $blogHtml);
                }
                $blogSourceType = isset($blogOptions['type']) ? $blogOptions['type'] : '';
                if ($blogSourceType === 'Tags') {
                    $blogSource = 'tags:' . (isset($blogOptions['tags']) && $blogOptions['tags'] ? $blogOptions['tags'] : '');
                } else {
                    $blogSource = isset($blogOptions['source']) && $blogOptions['source'] ? $blogOptions['source'] : '';
                }
                self::$posts = self::getBlogPosts($blogSource);
                return preg_replace_callback(
                    '/<\!--blog_post-->([\s\S]+?)<\!--\/blog_post-->/',
                    function ($postMatch) {
                        $postHtml = $postMatch[1];

                        if (count(self::$posts) < 1) {
                            return ''; // remove cell, if post is missing
                        }

                        self::$post = array_shift(self::$posts);
                        $postHtml = preg_replace_callback('/<\!--blog_post_header-->([\s\S]+?)<\!--\/blog_post_header-->/', 'self::setHeaderData', $postHtml);
                        $postHtml = preg_replace_callback('/<\!--blog_post_content-->([\s\S]+?)<\!--\/blog_post_content-->/', 'self::setContentData', $postHtml);
                        $postHtml = preg_replace_callback('/<\!--blog_post_image-->([\s\S]+?)<\!--\/blog_post_image-->/', 'self::setImageData', $postHtml);
                        $postHtml = preg_replace_callback('/<\!--blog_post_readmore-->([\s\S]+?)<\!--\/blog_post_readmore-->/', 'self::setReadmoreData', $postHtml);
                        $postHtml = preg_replace_callback('/<\!--blog_post_metadata-->([\s\S]+?)<\!--\/blog_post_metadata-->/', 'self::setMetadataData', $postHtml);
                        $postHtml = preg_replace_callback('/<\!--blog_post_tags-->([\s\S]+?)<\!--\/blog_post_tags-->/', 'self::setTagsData', $postHtml);
                        return $postHtml;
                    },
                    $blogHtml
                );
            },
            $content
        );

        return $content;
    }

    /**
     * Get blog post by source
     *
     * @param string $source Source
     *
     * @return array
     */
    public static function getBlogPosts($source) {
        $posts = array();
        $categoryId = '';
        $tags = '';
        if ($source) {
            if (preg_match('/^tags:/', $source)) {
                $tags = str_replace('tags:', '', $source);
            } else {
                $categoryObject = Nicepage_Data_Mappers::get('category');
                $categoryList = $categoryObject->find(array('title' => $source));
                if (count($categoryList) < 1) {
                    return $posts;
                }
                $categoryId = $categoryList[0]->id;
            }
        }
        // Get recent articles, if $categoryId is empty
        $blog = new ContentModelCustomArticles(array('category_id' => $categoryId, 'tags' => $tags));
        return $blog->getPosts();
    }

    /**
     * Set header
     *
     * @param string $headerMatch Header match
     *
     * @return mixed|string|string[]|null
     */
    public static function setHeaderData($headerMatch) {
        $headerHtml = $headerMatch[1];
        $headerHtml = preg_replace_callback(
            '/<\!--blog_post_header_content-->([\s\S]+?)<\!--\/blog_post_header_content-->/',
            function ($headerContentMatch) {
                return isset(self::$post['post-header']) ? self::$post['post-header'] : $headerContentMatch[1];
            },
            $headerHtml
        );
        $headerLink = isset(self::$post['post-header-link']) ? self::$post['post-header-link'] : '#';
        $headerHtml = preg_replace('/(href=[\'"])([\s\S]+?)([\'"])/', '$1' . $headerLink . '$3', $headerHtml);
        return $headerHtml;
    }

    /**
     * Set content
     *
     * @param string $contentMatch Content match
     *
     * @return mixed|string|string[]|null
     */
    public static function setContentData($contentMatch) {
        $contentHtml = $contentMatch[1];
        $contentHtml = preg_replace_callback(
            '/<\!--blog_post_content_content-->([\s\S]+?)<\!--\/blog_post_content_content-->/',
            function ($contentMatch) {
                return isset(self::$post['post-content']) ? self::$post['post-content'] : $contentMatch[1];
            },
            $contentHtml
        );
        return $contentHtml;
    }

    /**
     * Set image
     *
     * @param string $imageMatch Image match
     *
     * @return mixed
     */
    public static function setImageData($imageMatch) {
        $imageHtml = $imageMatch[1];
        $src = isset(self::$post['post-image'])? self::$post['post-image'] : '';
        if (!$src) {
            return '';
        }
        $imageHtml = preg_replace('/(src=[\'"])([\s\S]+?)([\'"])/', '$1' . self::$post['post-image'] . '$3', $imageHtml);
        return $imageHtml;
    }

    /**
     * Set readmore
     *
     * @param string $readmoreMatch Readmre match
     *
     * @return mixed|string|string[]|null
     */
    public static function setReadmoreData($readmoreMatch) {
        $readmoreHtml = $readmoreMatch[1];
        $readmoreHtml = preg_replace_callback(
            '/<\!--blog_post_readmore_content-->([\s\S]+?)<\!--\/blog_post_readmore_content-->/',
            function ($readmoreContentMatch) {
                return isset(self::$post['post-readmore-text']) ? self::$post['post-readmore-text'] : $readmoreContentMatch[1];
            },
            $readmoreHtml
        );
        $readmoreHtml = preg_replace_callback(
            '/<\!--blog_post_readmore_link-->([\s\S]+?)<\!--\/blog_post_readmore_link-->/',
            function ($readmoreLinkMatch) {
                return isset(self::$post['post-readmore-link']) ? self::$post['post-readmore-link'] : $readmoreLinkMatch[1];
            },
            $readmoreHtml
        );
        return $readmoreHtml;
    }

    public static $type = '';

    /**
     * Set metadata
     *
     * @param string $metadataMatch Metadata match
     *
     * @return mixed|string|string[]|null
     */
    public static function setMetadataData($metadataMatch) {
        $metadataHtml = $metadataMatch[1];
        $metaDataTypes = array('date', 'author', 'category', 'comments', 'edit');
        foreach ($metaDataTypes as $type) {
            self::$type = $type;
            $metadataHtml = preg_replace_callback(
                '/<\!--blog_post_metadata_' . self::$type . '-->([\s\S]+?)<\!--\/blog_post_metadata_' . self::$type . '-->/',
                function ($metadataTypeMatch) {
                    return $metadataTypeMatch[1];
                },
                $metadataHtml
            );
            $metadataHtml = preg_replace_callback(
                '/<\!--blog_post_metadata_' . self::$type . '_content-->([\s\S]+?)<\!--\/blog_post_metadata_' . self::$type . '_content-->/',
                function ($metadataTypeContentMatch) {
                    return isset(self::$post['post-metadata-' . self::$type]) ? self::$post['post-metadata-' . self::$type] : $metadataTypeContentMatch[1];
                },
                $metadataHtml
            );
        }
        return $metadataHtml;
    }

    /**
     * Set tags
     *
     * @param string $tagsMatch tags match
     *
     * @return mixed|string|string[]|null
     */
    public static function setTagsData($tagsMatch) {
        $tagsHtml = $tagsMatch[1];
        $tagsHtml = preg_replace_callback(
            '/<\!--blog_post_tags_content-->([\s\S]+?)<\!--\/blog_post_tags_content-->/',
            function ($contentTagsMatch) {
                return isset(self::$post['post-tags']) ? self::$post['post-tags'] : $contentTagsMatch[1];
            },
            $tagsHtml
        );
        return $tagsHtml;
    }

    public static $controlName = '';
    /**
     * Process all custom controls on the header
     *
     * @param string $content content
     *
     * @return mixed
     */
    public static function processControls($content) {
        $controls = ['menu', 'logo', 'headline', 'search', 'login'];
        foreach ($controls as $value) {
            self::$controlName = $value;
            $content =  preg_replace_callback(
                '/<\!--np_' . $value . '--><!--np_json-->([\s\S]+?)<\!--\/np_json-->([\s\S]*?)<\!--\/np_' . $value . '-->/',
                function ($matches) {
                    $controlProps = json_decode(trim($matches[1]), true);
                    $controlTemplate = $matches[2];
                    ob_start();
                    include JPATH_ADMINISTRATOR . '/components/com_nicepage/views/controls/'. self::$controlName . '/' . self::$controlName . '.php';
                    return ob_get_clean();
                },
                $content
            );
        }
        return $content;
    }

    /**
     * Process all custom php on the page
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processCustomPhp($content) {
        $content = preg_replace('/data-custom-php=["\']+(<\!--custom_php-->[\s\S]+?<\!--\/custom_php-->)["\']+([^>]*?>)/', '$2$1', $content);
        $content =  preg_replace_callback(
            '/<\!--custom_php-->([\s\S]+?)<\!--\/custom_php-->/',
            function ($matches) {
                $code = trim($matches[1]);
                $code = str_replace("<?php", "", $code);
                $code = str_replace("?>", "", $code);
                $code = str_replace("&quot;", "\"", $code);
                ob_start();
                @eval($code);
                return ob_get_clean();
            },
            $content
        );
        return $content;
    }

    /**
     * @var array All blocks on the page
     */
    public static $blockLayouts = array();

    /**
     * Process all positions on the page
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processPositions($content) {
        $content = preg_replace_callback(
            '/<\!--position-->([\s\S]+?)<\!--\/position-->/',
            function ($match) {
                $block = $match[1];
                preg_match('/data-position="([^"]*)"/', $block, $match2);
                $position = $match2[1];
                $i = count(self::$blockLayouts) + 1;
                self::$blockLayouts[$i] = $block;
                $document = JFactory::getDocument();
                if ($position && $document->countModules($position) !== 0) {
                    $attr = array(
                        'style' => 'upstylefromplugin',
                        'iterator' => $i,
                        'title' => 'name-' . $i,
                        'id' => $i,
                        'name' => $position
                    );
                    include_once dirname(dirname(__FILE__)) . '/helpers/modules.php';
                    return $document->getBuffer('modules', $position, $attr);
                } else {
                    return '<div class="hidden-position" style="display: none"></div>';
                }
            },
            $content
        );
        return $content;
    }

    /**
     * Processing of default image path
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processDefaultImage($content)
    {
        // replace default image placeholder
        $url = JURI::current() . 'components/com_nicepage/assets/images/nicepage-images/default-image.jpg';
        return str_replace('[image_default]', $url, $content);
    }

    /**
     * Processing of forms
     *
     * @param string $content Page content
     * @param string $pageId  Type id
     *
     * @return mixed
     */
    public static function processForm($content, $pageId)
    {
        // process source is joomla
        $content = preg_replace('/(<form[^>]*action=[\'\"]+).*?([\'\"][^>]*source=[\'\"]joomla)/', '$1index.php?option=com_nicepage&task=sendmail$2', $content);
        $content = preg_replace_callback(
            '/<form[^>]+redirect-address=[\'\"](.*?)[\'\"][^>]*>/',
            function ($matches) {
                if (preg_match('/<form[^>]+redirect=[\'\"]true[\'\"]/', $matches[0])) {
                    return $matches[0] . '<input type="hidden" name="redirect" value="' . $matches[1] . '">';
                }
                return $matches[0];
            },
            $content
        );
        // process source is customphp
        if ($pageId) {
            $content = preg_replace(
                '/(<form[^>]*action=[\'\"]+)\[\[form\-(.*?)\]\]([\'\"][^>]*source=[\'\"]customphp)/',
                '$1index.php?option=com_nicepage&task=form&id=' . $pageId . '&formId=$2$3',
                $content
            );
        }
        return $content;
    }

    /**
     * Prepare page props
     *
     * @param array  $props          Page props
     * @param array  $comConfig      Plg config
     * @param string $articleContext Article context
     * @param object $article        Article
     * @param object $articleParams  Article params
     *
     * @return mixed
     */
    public static function prepareProps($props, $comConfig, $articleContext, $article, $articleParams) {
        // Process image paths
        $props['publishHtml'] = self::fixImagePaths($props['publishHtml']);
        $props['head']        = self::fixImagePaths($props['head']);
        $props['bodyStyle']   = self::fixImagePaths($props['bodyStyle']);
        $props['fonts']       = self::fixImagePaths($props['fonts']);

        // Process backlink
        if ($comConfig) {
            $hideBacklink = isset($comConfig['hideBacklink']) ? (bool)$comConfig['hideBacklink'] : false;
            $backlink = $props['backlink'];
            $props['backlink'] = $hideBacklink ? str_replace('u-backlink', 'u-backlink u-hidden', $backlink) : $backlink;
        }

        // Process content
        $article->doubleСall = true;
        $currentText = $article->text;
        $currentPostId = $article->id;
        $article->text = $props['publishHtml'];
        $article->id = '-1';
        JPluginHelper::importPlugin('content');
        $dispatcher = JEventDispatcher::getInstance();
        $dispatcher->trigger('onContentPrepare', array($articleContext, &$article, &$articleParams, 0));
        $props['publishHtml'] = $article->text;
        $article->text = $currentText;
        $article->id = $currentPostId;

        $props['pageId']      = $article->id;
        $props['bodyClass']   = isset($props['bodyClass']) ? $props['bodyClass'] : '';
        $props['bodyStyle']   = isset($props['bodyStyle']) ? $props['bodyStyle'] : '';
        $props['head']        = isset($props['head']) ? $props['head'] : '';
        $props['fonts']       = isset($props['fonts']) ? $props['fonts'] : '';
        $props['publishHtml'] = isset($props['publishHtml']) ? $props['publishHtml'] : '';

        $props['backlink']       = isset($props['backlink']) ? $props['backlink'] : '';
        $props['pageCssUsedIds'] = isset($props['pageCssUsedIds']) ? $props['pageCssUsedIds'] : '';
        $props['hideHeader']     = isset($props['hideHeader']) ? $props['hideHeader'] : false;
        $props['hideFooter']     = isset($props['hideFooter']) ? $props['hideFooter'] : false;
        $props['hideBackToTop']  = isset($props['hideBackToTop']) ? $props['hideBackToTop'] : false;

        $props['metaTags']       = isset($props['metaTags']) ? $props['metaTags'] : '';
        $props['customHeadHtml'] = isset($props['customHeadHtml']) ? $props['customHeadHtml'] : '';
        $props['metaGeneratorContent'] = isset($props['metaGeneratorContent']) ? $props['metaGeneratorContent'] : '';
        $props['canonical'] = isset($props['canonical']) ? $props['canonical'] : '';

        return $props;
    }

    /**
     * Fix image paths
     *
     * @param string $content Content
     *
     * @return mixed
     */
    public static function fixImagePaths($content) {
        return str_replace('[[site_path_live]]', JURI::root(), $content);
    }
}